##
# This class is used to convert currencies and do some simple math.

class MoneyMaahd
  # reader and writer for both class and instance variables
  attr_accessor :amount, :currency, :base_currency, :rates

  # initialize with instance variables
  def initialize(amount, currency)
    @amount = amount
    @currency = currency
  end

  # override the inspect method
  def inspect
    '%.2f' % [@amount] + " " + @currency
  end

  # define class method for assigning base currency and exchange rates
  def self.conversion_rates(base_currency, rates = {})
    @@base_currency = base_currency
    @@rates = rates
  end

  # converts to a different currency
  def convert_to(currency)
    if currency != @@base_currency
      MoneyMaahd.new('%.2f' % [@amount * @@rates[currency]], currency)
    else
      MoneyMaahd.new('%.2f' % [@amount.to_f / @@rates[@currency]], currency)
    end
  end

  # override the plus operator
  def +(right)
    right_in_base_currency = right.convert_to(@@base_currency)
    sum = @amount + right_in_base_currency.amount.to_f
    MoneyMaahd.new('%.2f' % [sum], @@base_currency)
  end

  # override the subtraction operator
  def -(right)
    right_in_base_currency = right.convert_to(@@base_currency)
    difference = @amount - right_in_base_currency.amount.to_f
    MoneyMaahd.new('%.2f' % [difference], @@base_currency)
  end

  # override the division operator
  def /(right)
    quotient = @amount / right
    MoneyMaahd.new('%.2f' % [quotient], @currency)
  end

  # override the multiply operator
  def *(right)
    product = @amount * right
    MoneyMaahd.new('%.2f' % [product], @currency)
  end

  # override the is equal operator
  def ==(right)
    if right.currency == @currency
      @amount == right.amount
    else
      left = self.convert_to(@@base_currency)
      left_amount = left.amount.to_f
      left_amount == right.amount
    end
  end

  # override the is greater than operator
  def >(right)
    if self.currency == right.currency
      self.amount > right.amount
    else # if left is not base currency
      left = self.convert_to(@@base_currency)
      left_amount = left.amount.to_f
      left_amount > right.amount
    end
  end

  # override the is less than operator
  def <(right)
    if self.currency == right.currency
      self.amount < right.amount
    else #if left is not base currency
      left = self.convert_to(@@base_currency)
      left_amount = left.amount.to_f
      left_amount < right.amount
    end
  end
end
